/*
dtmf detection xmms plugin
Copyright (C) 2003 by Jon Snell <jsnell@networkninja.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


compilation directions:
 gcc -shared -o libdtmf.so -lgtk  -I/usr/include/gtk-1.2/  -I/usr/lib/glib/include -I/usr/include/glib-1.2 -I/usr/lib/gtk-1.2/include/ -I/usr/include/pango-1.0/ -I/usr/include/atk-1.0/ dtmf.c about.c -Wall

cp libdtmf.so /usr/lib/xmms/Effect/

*/

#include "dtmf.h"
#include "math.h"
#include <gtk/gtk.h>


static int mod_samples(gpointer * d, gint length, AFormat afmt, gint srate, gint nch);

static void init(void);
static void cleanup(void);
void gui_init();
void config();
GtkWidget *main_win=NULL, *text_box;

char last_digit = ' ';

EffectPlugin dtmf_ep =
{
	NULL,
	NULL,
	NULL, /* Description */
	init,
	cleanup,
	dtmf_about,
	config,
	mod_samples,
};

EffectPlugin *get_eplugin_info(void)
{
  dtmf_ep.description = g_strdup_printf(("DTMF Detection Plugin%s"), VERSION);
  return &dtmf_ep;
}

static void init(void)
{
  if (!main_win) gui_init();
  gtk_widget_show(main_win);
    
}

static void cleanup(void)
{
      


}


/* Create main GUI window */
void gui_init(){
	GtkWidget *hbox, *panel;

	if (main_win)return;
	main_win = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_signal_connect(GTK_OBJECT(main_win), "destroy",
			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &main_win);
	gtk_window_set_title(GTK_WINDOW(main_win), "Dialed Numbers");
	gtk_window_set_policy(GTK_WINDOW(main_win), FALSE, FALSE, FALSE);
	gtk_window_set_position(GTK_WINDOW(main_win), GTK_WIN_POS_MOUSE);

	hbox = gtk_hbox_new(FALSE, 1);
	gtk_container_add(GTK_CONTAINER(main_win), hbox);

	panel=gtk_handle_box_new();
	gtk_container_set_border_width(GTK_CONTAINER(panel),5);
	text_box = gtk_text_new (NULL, NULL);
	gtk_container_add(GTK_CONTAINER(panel),text_box);
	gtk_widget_show(text_box); 
	
	gtk_box_pack_start(GTK_BOX(hbox),panel, FALSE, FALSE,0);
	gtk_widget_show(panel);		
	
	gtk_widget_show(hbox);	
}


void config(){

}


static int mod_samples(gpointer * d, gint length, AFormat afmt, gint srate, gint nch)
{
  int x,xn;
  
  char buf[255];
  int freqs[8] = { 697, 770, 852, 941, 1209, 1336, 1477,1633};
  int freq_match[8], cur_freq;
    
  gint16 *dataptr = (gint16 *)*d,  *origptr = (gint16 *)*d;


  double q0, q1, q2, k, w, cos_w, sin_w, coeff, r_comp, imag, mag;
  int freq_c;
  char digit;
  
  if (!(afmt == FMT_S16_NE || (afmt == FMT_S16_LE && G_BYTE_ORDER == G_LITTLE_ENDIAN) || (afmt == FMT_S16_BE && G_BYTE_ORDER == G_BIG_ENDIAN)))
    return length; 
  

  xn = 0;
  dataptr = origptr;

  // goertzel implementation
  for(freq_c = 0; freq_c < 8; freq_c++)
    {
      freq_match[freq_c] = 0;
      dataptr = origptr;
      cur_freq = freqs[freq_c];
      q0 = q1 = q2 = 0;
      
      k = (int)(0.5+(length*cur_freq/(float)srate));
      w = (2*M_PI / length)* k;
      cos_w = cos(w);
      sin_w = sin(w);
      coeff = 2 * cos_w;  
      
      for (x = 0; x < length; x += (nch == 2 ? 4 : 2))
	{
	  q0 = coeff * q1 - q2 + dataptr[0];   
	  q2 = q1;
	  q1 = q0;
	  dataptr+=1;
	  if (nch == 2)
	    dataptr+=1;     
	}
      
      r_comp = (q1 - q2 * cos_w);
      imag = (q2 * sin_w);
      mag =pow( pow(r_comp, 2) + pow(imag, 2), 0.5);
      if (mag > 2000000)
	freq_match[freq_c] = 1;
    }

  // now determine what pairs we've got
  digit = ' ';
  // this tends to favor the last digits
  if (freq_match[0] && freq_match[4])
    digit = '1';
  if (freq_match[1] && freq_match[4])
    digit = '4';
  if (freq_match[2] && freq_match[4])
    digit = '7';
  if (freq_match[3] && freq_match[4])
    digit = '*';
  if (freq_match[0] && freq_match[5])
    digit = '2';
  if (freq_match[1] && freq_match[5])
    digit = '5';
  if (freq_match[2] && freq_match[5])
    digit = '8';
  if (freq_match[3] && freq_match[5])
    digit = '0';
  if (freq_match[0] && freq_match[6])
    digit = '3';
  if (freq_match[1] && freq_match[6])
    digit = '6';
  if (freq_match[2] && freq_match[6])
    digit = '9';
  if (freq_match[3] && freq_match[6])
    digit = '#';
  if (freq_match[0] && freq_match[7])
    digit = 'A';
  if (freq_match[1] && freq_match[7])
    digit = 'B';
  if (freq_match[2] && freq_match[7])
    digit = 'C';
  if (freq_match[3] && freq_match[7])
    digit = 'D';

  if (digit != ' ')
    {
      if (digit != last_digit)
	{
	  sprintf(buf, "%c", digit);
	  gtk_text_insert (GTK_TEXT (text_box), NULL, &text_box->style->black, NULL,  buf, -1);
	}
    }
  last_digit = digit;
  return length;
	
}
